# -*- coding: utf-8 -*-
"""
Created on Fri Oct 20 10:46:36 2023

Author: vlandgraf
"""

import gemdat
from pathlib import Path
from gemdat import SitesData, Trajectory, simulation_metrics
from gemdat.io import load_known_material
from pymatgen.io.vasp import Vasprun
from pymatgen.io.vasp.outputs import Poscar
import general_AF_structure_analysis
import pickle as pkl
import numpy as np
import glob

# Constants
k_b = 8.6173E-5  # Boltzmann constant in eV/K

# Directories for LSN supercells
folders_with_LSN_vaspruns = [None]  # Replace None with actual directory names

# Load LSN library template
with open('./jump_lib_LSN_template.pkl', 'rb') as h: 
    lsn_lib = pkl.load(h)

# Initialize fields in the LSN library
for k, v in lsn_lib.items():
    lsn_lib[k]['type_of_cell'] = []

# Lists to store occupancy data
tet_occupancy = []
oct_occupancy = []

# Process each folder
for cell in folders_with_LSN_vaspruns: 
    # Get trajectory file
    files_in_dir = glob.glob(cell + '/*vasprun.xml*')[0]
    trajectory = Trajectory.from_cache(files_in_dir)
    
    # Get the initial structure and save it
    s = trajectory.get_structure(0)  
    Poscar(s).write_file(cell + '/POSCAR')
    
    # Get structure with all Li atoms
    s = general_AF_structure_analysis.get_structure_with_all_Li(s)
    
    # Create SitesData object
    sites = SitesData(
        structure=s,
        trajectory=trajectory,
        floating_specie='Li', n_parts=1
    )
    
    # Get temperature in K
    temp = trajectory.metadata['temperature']
    
    # Getting transitions and occupancy matrices
    transitions = np.array([row[:96] for row in sites.transitions_parts[0][:96]])
    occupancy = np.array([i for i in sites.occupancy_parts[0].values()][:96])
    
    oct_occupancy.append(np.sum(occupancy[:32]))
    tet_occupancy.append(np.sum(occupancy[32:]))
    
    # Get the structure with polyhedra
    s_with_polyhedra = general_AF_structure_analysis.find_tet_for_Li(s)

    # Create dictionaries for site-pair types
    tet_tet_dict = general_AF_structure_analysis.make_pair_dictionary_tet_tet(s_with_polyhedra)
    oct_tet_dict = general_AF_structure_analysis.make_pair_dictionary_oct_tet(s_with_polyhedra)
    tet_oct_dict = general_AF_structure_analysis.make_pair_dictionary_tet_oct(s_with_polyhedra)
    
    # Calculate activation energies
    activation_energies = np.log(transitions / (occupancy[:, np.newaxis] * 2E-15 * 1E13)) * k_b * temp * -1
    
    # Update the LSN library with activation energies
    for i in range(96): 
        if i < 32:  # Octahedral site as the starting site
            for j in range(32, 96):  # Tetrahedral site as the ending site
                if 0.05 < activation_energies[i][j] < 5 and (i, j) in oct_tet_dict:
                    key = oct_tet_dict[(i, j)]['jump_type_precise']
                    lsn_lib[key]['activation_energies'].append(activation_energies[i][j])
                    average = np.average(lsn_lib[key]['activation_energies'])
					length_of_averages = len(lsn_lib[key]['activation_energies'])
                    error = np.std(lsn_lib[key]['activation_energies']) / np.sqrt(length_of_averages)
                    lsn_lib[key]['average_ea'] = average
                    lsn_lib[key]['error_in_ea'] = error  
                    lsn_lib[key]['type_of_cell'].append(s_with_polyhedra.composition)
        
        else:  # Tetrahedral site as the starting site
            # Octahedral site as the ending site
            for j in range(0, 32):
                if 0.05 < activation_energies[i][j] < 5 and (i, j) in tet_oct_dict:
                    key = tet_oct_dict[(i, j)]['jump_type_precise']
                    lsn_lib[key]['activation_energies'].append(activation_energies[i][j])
                    average = np.average(lsn_lib[key]['activation_energies'])
					length_of_averages = len(lsn_lib[key]['activation_energies'])
                    error = np.std(lsn_lib[key]['activation_energies']) / np.sqrt(length_of_averages)
                    lsn_lib[key]['average_ea'] = average
                    lsn_lib[key]['error_in_ea'] = error  
                    lsn_lib[key]['type_of_cell'].append(s_with_polyhedra.composition)
            
            # Tetrahedral site as the ending site
            for j in range(32, 96):
                if 0.05 < activation_energies[i][j] < 5 and (i, j) in tet_tet_dict:
                    key = tet_tet_dict[(i, j)]['jump_type_precise']
                    lsn_lib[key]['activation_energies'].append(activation_energies[i][j])
                    average = np.average(lsn_lib[key]['activation_energies'])
					length_of_averages = len(lsn_lib[key]['activation_energies'])
                    error = np.std(lsn_lib[key]['activation_energies']) / np.sqrt(length_of_averages)
                    lsn_lib[key]['average_ea'] = average
                    lsn_lib[key]['error_in_ea'] = error  
                    lsn_lib[key]['type_of_cell'].append(s_with_polyhedra.composition)

# Add placeholders for jumps without site-pairs represented in the supercell
for k, v in lsn_lib.items():
    if not lsn_lib[k]['activation_energies']:
        print(f"Key with no activation energies: {k}")
        lsn_lib[k]['average_ea'] = 0.74

# Save results
with open('jump_lib_LSN.pkl', 'wb') as h: 
    pkl.dump(lsn_lib, h)
